package com.gx.obe.struct.composite;

import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.bind.prop.Prop;
import com.gx.obe.component.handler.TabHandler;
import com.gx.obe.struct.inters.IModelDataDetail;
import com.gx.obe.struct.widget.TabButton;
import com.swtdesigner.SWTResourceManager;

public class ModelDataDetailComposite extends Composite {
	
	private final Prop<String> titleName = new Prop<>();
	
	private final IModelDataDetail modelDataDetail;
	
	public ModelDataDetailComposite(Composite parent, String tenderId) {
		super(parent, SWT.NONE);
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		Composite composite = new Composite(this, SWT.NONE);
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.marginHeight = 10;
		composite.setLayout(gl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Label lblNewLabel = new Label(composite, SWT.NONE);
		lblNewLabel.setFont(SWTResourceManager.getFont("Microsoft YaHei UI", 10, SWT.BOLD));
		lblNewLabel.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1));
		titleName.bind(t -> {
			lblNewLabel.setText(t);
			lblNewLabel.getParent().layout();
		});
		
		ESepator label = new ESepator(this, SWT.HORIZONTAL);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		label.setBackground(SWTResourceManager.getColor(222, 234, 245));
		label.setLineWidth(2);
		
		Composite composite_1 = new Composite(this, SWT.NONE);
		GridLayout gl_composite_1 = new GridLayout(1, false);
		gl_composite_1.verticalSpacing = 0;
		gl_composite_1.marginHeight = 0;
		gl_composite_1.marginWidth = 0;
		composite_1.setLayout(gl_composite_1);
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		Composite composite_2 = new Composite(composite_1, SWT.NONE);
		RowLayout rl_composite_2 = new RowLayout(SWT.HORIZONTAL);
		rl_composite_2.marginBottom = 0;
		rl_composite_2.marginTop = 0;
		rl_composite_2.marginRight = 0;
		rl_composite_2.marginLeft = 0;
		rl_composite_2.spacing = 0;
		composite_2.setLayout(rl_composite_2);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		TabHandler<TabButton> handler = new TabHandler<>(composite_2, TabButton::new);
		
		ESepator label_1 = new ESepator(composite_1, SWT.HORIZONTAL);
		label_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		label_1.setBackground(SWTResourceManager.getColor(222, 234, 245));
		
		ScrolledComposite scrolledComposite = new ScrolledComposite(composite_1, SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);
		
		Composite composite_3 = new Composite(scrolledComposite, SWT.NONE);
		GridLayout gl_composite_3 = new GridLayout(1, false);
		gl_composite_3.marginHeight = 15;
		gl_composite_3.marginWidth = 0;
		composite_3.setLayout(gl_composite_3);
		
		StackLayout stackLayout = new StackLayout();
		
		Composite composite_4 = new Composite(composite_3, SWT.NONE);
		composite_4.setLayout(stackLayout);
		GridData gd_composite_4 = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_composite_4.widthHint = 700;
		composite_4.setLayoutData(gd_composite_4);
		scrolledComposite.setContent(composite_3);
		scrolledComposite.setMinSize(composite_3.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		
		modelDataDetail = new IModelDataDetail() {
			
			@Override
			public void refreshTitle(String title) {
				titleName.set(title);
			}
			
			public void addBasicSubpage(Function<Composite, Composite> initSub) {
				handler.start();
				addSubpage("基本信息", initSub);
			}
			
			public void addSubpage(String name, Function<Composite, Composite> initSub) {
				Composite subComposite = initSub.apply(composite_4);
				handler.hand(name, () -> {
					stackLayout.topControl = subComposite;
					composite_4.layout();
					composite_3.layout();
					scrolledComposite.setContent(composite_3);
					scrolledComposite.setMinSize(composite_3.computeSize(SWT.DEFAULT, SWT.DEFAULT));
				});
			}
			
			public <T> void addAttachmentSubpage(Consumer<BiConsumer<String, String>> supplierIdAndBusinessIdConsumer) {
				addSubpage("附件列表", p -> {
					AttachmentComposite attachmentComposite = new AttachmentComposite(p, tenderId);
					supplierIdAndBusinessIdConsumer.accept((supplierId, businessId) -> {
						attachmentComposite.refresh(supplierId, businessId);
					});
					return attachmentComposite;
				});
				handler.end();
			}
			
		};
		
	}
	
	public IModelDataDetail getModelDataDetail() {
		return modelDataDetail;
	}
	
	@Override
	protected void checkSubclass() {}
}
